/*-------------------------------------------------------------------------+
| Copyright 2013 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.rcp.application.actions;

import static org.fortiss.af3.project.storage.LocationProvider.CURRENT_AF3_PROJECT_FILES_EXTENSION;
import static org.fortiss.af3.project.ui.utils.NewProjectUtils.createNewGeneralProject;
import static org.fortiss.af3.project.ui.utils.NewProjectUtils.generalProjectExists;
import static org.fortiss.af3.rcp.application.utils.ApplicationUtils.refreshWorkspace;
import static org.fortiss.af3.rcp.application.utils.ApplicationUtils.saveProject;

import java.util.Properties;

import org.eclipse.core.resources.IFile;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.intro.IIntroPart;
import org.eclipse.ui.intro.IIntroSite;
import org.eclipse.ui.intro.config.IIntroAction;
import org.fortiss.af3.project.model.FileProject;
import org.fortiss.af3.project.utils.ProjectUtils;
import org.fortiss.af3.rcp.application.utils.ApplicationUtils;

/**
 * Base class for loading standard examples.
 * 
 * @author wenwenchen
 */
public abstract class LoadExampleActionBase implements IIntroAction {

	/** The current AF3 model file extension. */
	private static final String CURRENT_AF3_FILE_EXTENSION =
			"." + CURRENT_AF3_PROJECT_FILES_EXTENSION;

	/** {@inheritDoc} */
	@Override
	public void run(IIntroSite site, Properties params) {
		final IIntroPart introPart = PlatformUI.getWorkbench().getIntroManager().getIntro();
		PlatformUI.getWorkbench().getIntroManager().setIntroStandby(introPart, true);

		startup();

		loadExample(CURRENT_AF3_FILE_EXTENSION);
	}

	/**
	 * Method that is used by multiple command handlers to ensure that the
	 * Project exists that contains the *.af3_2x files and that the
	 * AF3-Perspective is activated.
	 */
	private void startup() {
		// start up
		if(!generalProjectExists()) {
			// Create a generic project.
			createNewGeneralProject();
		}
		// Enable the AF3 Perspective.
		ApplicationUtils.switchToAF3Perspective();
	}

	/** load example according to example name */
	protected abstract void loadExample(String suffix);

	/**
	 * Loads an existing *.af3_2x Example file from a given path and adds it to
	 * the Model-Navigator.
	 * 
	 * @param pluginID
	 *            the plugin ID
	 * @param relativePath
	 *            the relative path to the file
	 * @param projectName
	 *            the name of the file
	 * @param suffix
	 *            the suffix of the file
	 */
	protected synchronized static void loadExistingModel(String pluginID, String relativePath,
			String projectName, String suffix) {

		// Load file of the Example (do not create dummy, because
		// otherwise the Example is shown twice).
		FileProject project = ProjectUtils.loadProjectFromPlugin(pluginID, relativePath, false);

		// Check if file name not already exists.
		IFile targetProjectFile = ApplicationUtils.validateProjectName(projectName, suffix);

		if(targetProjectFile != null) {
			project.setName(targetProjectFile.getName().replace(suffix, ""));
			// Save the project to workspace.
			saveProject(project, targetProjectFile.getFullPath().toString());
			refreshWorkspace();
		}
	}
}
